@@ -31,6 +31,12 @@
 
 #include <libnautilus-extension/nautilus-extension-types.h>
 
+static const GDebugKey nautilus_python_debug_keys[] = {
+  {"misc", NAUTILUS_PYTHON_DEBUG_MISC},
+};
+static const guint nautilus_python_ndebug_keys = sizeof (nautilus_python_debug_keys) / sizeof (GDebugKey);
+NautilusPythonDebug nautilus_python_debug;
+
 static GArray *all_types = NULL;
 
 #define ENTRY_POINT "nautilus_extension_types"
@@ -82,12 +88,19 @@ nautilus_python_load_dir (GTypeModule *module, const char *dirname)
 {
 	GDir *dir;
 	const char *name;
+	PyObject *sys_path, *py_path;
 
 	debug_enter_args("dirname=%s", dirname);
 	
 	dir = g_dir_open(dirname, 0, NULL);
 	if (!dir)
 		return;
+
+	  /* sys.path.insert(0, dirname) */
+	sys_path = PySys_GetObject("path");
+	py_path = PyString_FromString(dirname);
+	PyList_Insert(sys_path, 0, py_path);
+	Py_DECREF(py_path);
 			
 	while ((name = g_dir_read_name(dir))) {
 		if (g_str_has_suffix(name, ".py")) {
@@ -107,13 +120,18 @@ gboolean
 nautilus_python_init_python (gchar **user_extensions_dir)
 {
 	PyObject *pygtk, *mdict, *require;
-	PyObject *sys_path, *nautilus, *gtk, *pygtk_version, *pygtk_required_version;
+	PyObject *sys_path, *tmp, *nautilus, *gtk, *pygtk_version, *pygtk_required_version;
+	GModule *libpython;
 	char *home_dir;
 	char *argv[] = { "nautilus", NULL };
 
 	if (Py_IsInitialized())
 		return TRUE;
-	
+
+	libpython = g_module_open("libpython" PYTHON_VERSION "." G_MODULE_SUFFIX, 0);
+	if (!libpython)
+		g_warning("g_module_open libpython failed: %s", g_module_error());
+
 	Py_Initialize();
 	PySys_SetArgv(1, argv);
 
@@ -161,9 +179,9 @@ nautilus_python_init_python (gchar **user_extensions_dir)
 							   g_get_home_dir());
 	*user_extensions_dir = home_dir;
 	PyList_Insert(sys_path, 0,
-				  PyString_FromString(NAUTILUS_LIBDIR "/nautilus-python"));
-	PyList_Insert(sys_path, 0, PyString_FromString(home_dir));
-			
+				  (tmp = PyString_FromString(NAUTILUS_LIBDIR "/nautilus-python")));
+	Py_DECREF(tmp);
+	
 	/* import nautilus */
 	g_setenv("INSIDE_NAUTILUS_PYTHON", "", FALSE);
 	debug("import nautilus");
@@ -200,6 +218,15 @@ void
 nautilus_module_initialize(GTypeModule *module)
 {
 	gchar *user_extensions_dir;
+	const gchar *env_string;
+
+	env_string = g_getenv("NAUTILUS_PYTHON_DEBUG");
+	if (env_string != NULL) {
+		nautilus_python_debug = g_parse_debug_string(env_string,
+													 nautilus_python_debug_keys,
+													 nautilus_python_ndebug_keys);
+		env_string = NULL;
+    }
 	
 	debug_enter();
 
@@ -217,12 +244,15 @@ void
 nautilus_module_shutdown(void)
 {
 	debug_enter();
-
+	  /* Shutting down Python triggers a bug that causes nautilus to
+	   * crash; better not to open this can of worms... */
+#if 0
 	if (Py_IsInitialized()) {
 		Py_Finalize();
 	}
 
 	debug("Shutting down nautilus-python extension.");
+#endif
 }
 
 void 
